import React, { ReactNode } from "react"
import { Binary, Info, Cuboid, Cog, ShieldCheck } from "lucide-react"
import { Input } from "../components/ui/input"

import {
    Accordion,
    AccordionContent,
    AccordionItem,
    AccordionTrigger,
} from "../components/ui/accordion"

import {
    useChainSpecV1,
    useBlocks,
    useSystemHealth
} from "../hooks"

type RowType = {
    title: string
    element: ReactNode
}

interface ItemProps {
    value: string
    elements: RowType[]
    icon: ReactNode
}

const Item: React.FC<ItemProps> = ({ value, elements, icon }) => {
    return (
        <AccordionItem className="bg-muted rounded px-4" value={value}>
            <AccordionTrigger>
                <div className="flex items-center gap-2 space-x-2 cursor-pointer">
                    {icon} {value}
                </div>
            </AccordionTrigger>
            <AccordionContent className="flex flex-col gap-2">
                {elements.map(({ title, element }: RowType, i: number) => (
                    <Row key={title} title={title} element={element} />
                ))}
            </AccordionContent>
        </AccordionItem>
    )
}

interface RowProps {
    title: string
    element: ReactNode
}

const Row: React.FC<RowProps> = ({ title, element }) => {
    return (
        <div className="flex flex-row gap-2 justify-between items-center">
            <div className="text-sm">{title}</div>
            {element}
        </div>
    )
}

export const HealthCheck = () => {
    const chainSpec = useChainSpecV1()
    const blocks = useBlocks()
    const health = useSystemHealth()

    const metadataElements = [
        {
            title: "Chain Name",
            element: <Input
                readOnly
                aria-label="Chain Name"
                type="text"
                className="w-[300px]"
                placeholder={chainSpec?.chainName}
            />
        },
        {
            title: "Genesis Hash",
            element: <Input
                readOnly
                aria-label="Genesis Hash"
                type="text"
                className="w-[300px]"
                placeholder={chainSpec?.genesisHash}
            />
        }
    ]
    const peersElements = [
        {
            title: "Sync State",
            element: <Input
                readOnly
                aria-label="Sync State"
                type="text"
                className="w-[300px]"
                placeholder={!health  ? "" : health.isSyncing ? "Syncing..." : "Synced"}
            />
        },
        {
            title: "Peers Connected",
            element: <Input
                readOnly
                aria-label="Peers Connected"
                type="text"
                className="w-[300px]"
                placeholder={health?.peers}
            />
        }
    ]
    const bestBlockElements = [
        {
            title: "Block Number",
            element: <Input
                readOnly
                aria-label="Block Number"
                type="text"
                className="w-[300px]"
                placeholder={blocks?.latest?.number}
            />
        },
        {
            title: "Block Hash",
            element: <Input
                readOnly
                aria-label="Block Hash"
                type="text"
                className="w-[300px]"
                placeholder={blocks?.latest?.hash}
            />
        },
        {
            title: "Parent Hash",
            element: <Input
                readOnly
                aria-label="Parent Hash"
                type="text"
                className="w-[300px]"
                placeholder={blocks?.latest?.parent}
            />
        }
    ]
    const finalizedBlockElements = [
        {
            title: "Block Number",
            element: <Input
                readOnly
                aria-label="Block Number"
                type="text"
                className="w-[300px]"
                placeholder={blocks?.finalized?.number}
            />
        },
        {
            title: "Block Hash",
            element: <Input
                readOnly
                aria-label="BlockHash"
                type="text"
                className="w-[300px]"
                placeholder={blocks?.finalized?.hash}
            />
        },
        {
            title: "Parent Hash",
            element: <Input
                readOnly
                aria-label="Parent Hash"
                type="text"
                className="w-[300px]"
                placeholder={blocks?.finalized?.parent}
            />
        }
    ]
    const coinElements = [
        {
            title: "Symbol",
            element: <Input
                readOnly
                aria-label="Coin Symbol"
                type="text"
                className="w-[300px]"
                placeholder={chainSpec?.properties?.tokenSymbol}
            />
        },
        {
            title: "Decimals",
            element: <Input
                readOnly
                aria-label="Block Hash"
                type="text"
                className="w-[300px]"
                placeholder={chainSpec?.properties?.tokenDecimals?.toString() ?? ""}
            />
        },
        {
            title: "Base58 Prefix",
            element: <Input
                readOnly
                aria-label="Base58 Prefix"
                type="text"
                className="w-[300px]"
                placeholder={chainSpec?.properties?.ss58Format?.toString() ?? ""}
            />
        }
    ]

    return (
        <Accordion type="single" defaultValue="Chain metadata" className="w-[500px] h-fit flex flex-col flex-1 gap-4 justify-center self-center">
            <Item value="Chain metadata" elements={metadataElements} icon={<Info className="w-4 h-4" />} />
            <Item value="Peers information" elements={peersElements} icon={<Binary className="w-4 h-4" />} />
            <Item value="Latest block" elements={bestBlockElements} icon={<Cuboid className="w-4 h-4" />} />
            <Item value="Finalized block" elements={finalizedBlockElements} icon={<ShieldCheck className="w-4 h-4" />} />
            <Item value="Coin properties" elements={coinElements} icon={<Cog className="w-4 h-4" />} />
        </Accordion>
    )
}
